@@ -18,7 +18,7 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: capture.c,v 1.18 2005/05/25 02:27:16 mschimek Exp $ */
+/* $Id: capture.c,v 1.19 2005/06/11 22:13:11 mschimek Exp $ */
 
 #undef NDEBUG
 
@@ -32,6 +32,7 @@
 #include <errno.h>
 #include <assert.h>
 #include <unistd.h>
+#include <limits.h>
 #ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
 #endif
@@ -480,8 +481,9 @@ mainloop(void)
 			binary_sliced(sliced, timestamp, lines);
 		if (bin_pes || bin_ts) {
 			/* XXX shouldn't use system time. */
-			pts = timestamp * 90000;
-			_vbi_dvb_mux_mux(mx, pts, sliced, lines, -1);
+			pts = (int64_t)(timestamp * 90000.0);
+			_vbi_dvb_mux_feed (mx, pts, sliced, lines,
+					   /* service_set: all */ -1);
 		}
 	}
 }
